package cn.axzo.msg.center.message.service;

import cn.axzo.msg.center.message.domain.dto.GroupTreeNodePathDTO;
import cn.axzo.msg.center.service.dto.GroupTreeNodeDTO;

import java.util.List;
import java.util.Optional;

/**
 * 消息分类树节点缓存管理
 *
 * @author cold_blade
 * @date 2023/10/14
 * @version 1.0
 */
public interface MessageGroupTreeNodeCacheService {

    /**
     * 获取所有有效的分类结点树
     *
     * @return 树列表
     */
    List<GroupTreeNodeDTO> listAllGroupTree();

    /**
     * 获取指定结点所在的树的根节点
     *
     * @param rootNodeCode 结点编码
     * @return 根节点
     */
    Optional<GroupTreeNodeDTO> queryRootNode(String rootNodeCode);

    /**
     * 根据结点编码查询结点信息
     *
     * @param groupNodeCode 结点编码
     * @return 结点信息
     */
    Optional<GroupTreeNodeDTO> queryNode(String groupNodeCode);

    /**
     * 获取叶结点对应的树的路径
     *
     * @param leafNodeCode 叶结点编码
     * @return 路径
     */
    Optional<GroupTreeNodePathDTO> queryLeafNodePath(String leafNodeCode);

    /**
     * 获取根节点对应的所有叶结点对应的树的路径
     *
     * @param rootNodeCode 根结点编码
     * @return 路径
     */
    List<GroupTreeNodePathDTO> listLeafNodePaths(String rootNodeCode);

    /**
     * 刷新缓存
     */
    void refreshCache();
}
